@@ -1017,11 +1017,9 @@ pub(crate) fn get_ro_region(ro_section: &Section, elf: &[u8]) -> MemoryRegion {
     // If offset > 0, the region will start at MM_PROGRAM_START + the offset of
     // the first read only byte. [MM_PROGRAM_START, MM_PROGRAM_START + offset)
     // will be unmappable, see MemoryRegion::vm_to_host.
-    MemoryRegion::new_from_slice(
+    MemoryRegion::new_readonly(
         ro_data,
         ebpf::MM_PROGRAM_START.saturating_add(offset as u64),
-        0,
-        false,
     )
 }
 
@@ -1032,7 +1030,7 @@ mod test {
         ebpf,
         elf::scroll::Pwrite,
         fuzz::fuzz,
-        syscalls::{BpfSyscallContext, BpfSyscallString, BpfSyscallU64},
+        syscalls::{BpfSyscallString, BpfSyscallU64},
         user_error::UserError,
         vm::{SyscallObject, TestInstructionMeter},
     };
@@ -1043,18 +1041,10 @@ mod test {
     fn syscall_registry() -> SyscallRegistry {
         let mut syscall_registry = SyscallRegistry::default();
         syscall_registry
-            .register_syscall_by_name(
-                b"log",
-                BpfSyscallString::init::<BpfSyscallContext, UserError>,
-                BpfSyscallString::call,
-            )
+            .register_syscall_by_name(b"log", BpfSyscallString::call)
             .unwrap();
         syscall_registry
-            .register_syscall_by_name(
-                b"log_64",
-                BpfSyscallU64::init::<BpfSyscallContext, UserError>,
-                BpfSyscallU64::call,
-            )
+            .register_syscall_by_name(b"log_64", BpfSyscallU64::call)
             .unwrap();
         syscall_registry
     }
@@ -1810,6 +1800,6 @@ mod test {
             Executable::jit_compile(&mut executable).unwrap();
         }
 
-        assert_eq!(18640, executable.mem_size());
+        assert_eq!(18616, executable.mem_size());
     }
 }
