@@ -20,8 +20,6 @@
 //! value. Hence some syscalls have unused arguments, or return a 0 value in all cases, in order to
 //! respect this convention.
 
-extern crate libc;
-
 use crate::{
     error::EbpfError,
     memory_region::{AccessType, MemoryMapping},
@@ -29,12 +27,8 @@ use crate::{
     user_error::UserError,
     vm::SyscallObject,
 };
-use libc::c_char;
 use std::{slice::from_raw_parts, str::from_utf8, u64};
 
-/// Test syscall context
-pub type BpfSyscallContext = u64;
-
 /// Return type of syscalls
 pub type Result = std::result::Result<u64, EbpfError<UserError>>;
 
@@ -86,12 +80,6 @@ pub const BPF_TRACE_PRINTK_IDX: u32 = 6;
 /// This would equally print the three numbers in `/sys/kernel/debug/tracing` file each time the
 /// program is run.
 pub struct BpfTracePrintf {}
-impl BpfTracePrintf {
-    /// new
-    pub fn init<C, E>(_unused: C) -> Box<dyn SyscallObject<UserError>> {
-        Box::new(Self {})
-    }
-}
 impl SyscallObject<UserError> for BpfTracePrintf {
     fn call(
         &mut self,
@@ -140,12 +128,6 @@ impl SyscallObject<UserError> for BpfTracePrintf {
 /// assert_eq!(result.unwrap(), 0x1122334455);
 /// ```
 pub struct BpfGatherBytes {}
-impl BpfGatherBytes {
-    /// new
-    pub fn init<C, E>(_unused: C) -> Box<dyn SyscallObject<UserError>> {
-        Box::new(Self {})
-    }
-}
 impl SyscallObject<UserError> for BpfGatherBytes {
     fn call(
         &mut self,
@@ -179,24 +161,18 @@ impl SyscallObject<UserError> for BpfGatherBytes {
 /// use solana_rbpf::vm::{Config, SyscallObject};
 /// use solana_rbpf::user_error::UserError;
 ///
-/// let val = vec![0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x22, 0x33];
+/// let mut val = &mut [0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x22, 0x33];
 /// let val_va = 0x100000000;
 ///
 /// let mut result: Result = Ok(0);
 /// let config = Config::default();
-/// let memory_mapping = MemoryMapping::new::<UserError>(vec![MemoryRegion::default(), MemoryRegion::new_from_slice(&val, val_va, 0, true)], &config).unwrap();
+/// let memory_mapping = MemoryMapping::new::<UserError>(vec![MemoryRegion::default(), MemoryRegion::new_writable(val, val_va)], &config).unwrap();
 /// BpfMemFrob::call(&mut BpfMemFrob {}, val_va, 8, 0, 0, 0, &memory_mapping, &mut result);
-/// assert_eq!(val, vec![0x2a, 0x2a, 0x2a, 0x2a, 0x2a, 0x3b, 0x08, 0x19]);
+/// assert_eq!(val, &[0x2a, 0x2a, 0x2a, 0x2a, 0x2a, 0x3b, 0x08, 0x19]);
 /// BpfMemFrob::call(&mut BpfMemFrob {}, val_va, 8, 0, 0, 0, &memory_mapping, &mut result);
-/// assert_eq!(val, vec![0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x22, 0x33]);
+/// assert_eq!(val, &[0x00, 0x00, 0x00, 0x00, 0x00, 0x11, 0x22, 0x33]);
 /// ```
 pub struct BpfMemFrob {}
-impl BpfMemFrob {
-    /// new
-    pub fn init<C, E>(_unused: C) -> Box<dyn SyscallObject<UserError>> {
-        Box::new(Self {})
-    }
-}
 impl SyscallObject<UserError> for BpfMemFrob {
     fn call(
         &mut self,
@@ -219,45 +195,6 @@ impl SyscallObject<UserError> for BpfMemFrob {
     }
 }
 
-/// Compute and return the square root of argument 1, cast as a float. Arguments 2 to 5 are
-/// unused.
-///
-/// # Examples
-///
-/// ```
-/// use solana_rbpf::syscalls::{BpfSqrtI, Result};
-/// use solana_rbpf::memory_region::{MemoryRegion, MemoryMapping};
-/// use solana_rbpf::vm::{Config, SyscallObject};
-/// use solana_rbpf::user_error::UserError;
-///
-/// let mut result: Result = Ok(0);
-/// let config = Config::default();
-/// let memory_mapping = MemoryMapping::new::<UserError>(vec![], &config).unwrap();
-/// BpfSqrtI::call(&mut BpfSqrtI {}, 9, 0, 0, 0, 0, &memory_mapping, &mut result);
-/// assert_eq!(result.unwrap(), 3);
-/// ```
-pub struct BpfSqrtI {}
-impl BpfSqrtI {
-    /// new
-    pub fn init<C, E>(_unused: C) -> Box<dyn SyscallObject<UserError>> {
-        Box::new(Self {})
-    }
-}
-impl SyscallObject<UserError> for BpfSqrtI {
-    fn call(
-        &mut self,
-        arg1: u64,
-        _arg2: u64,
-        _arg3: u64,
-        _arg4: u64,
-        _arg5: u64,
-        _memory_mapping: &MemoryMapping,
-        result: &mut Result,
-    ) {
-        *result = Result::Ok((arg1 as f64).sqrt() as u64);
-    }
-}
-
 /// C-like `strcmp`, return 0 if the strings are equal, and a non-null value otherwise.
 ///
 /// # Examples
@@ -275,21 +212,15 @@ impl SyscallObject<UserError> for BpfSqrtI {
 ///
 /// let mut result: Result = Ok(0);
 /// let config = Config::default();
-/// let memory_mapping = MemoryMapping::new::<UserError>(vec![MemoryRegion::default(), MemoryRegion::new_from_slice(foo.as_bytes(), va_foo, 0, false)], &config).unwrap();
+/// let memory_mapping = MemoryMapping::new::<UserError>(vec![MemoryRegion::default(), MemoryRegion::new_readonly(foo.as_bytes(), va_foo)], &config).unwrap();
 /// BpfStrCmp::call(&mut BpfStrCmp {}, va_foo, va_foo, 0, 0, 0, &memory_mapping, &mut result);
 /// assert!(result.unwrap() == 0);
 /// let mut result: Result = Ok(0);
-/// let memory_mapping = MemoryMapping::new::<UserError>(vec![MemoryRegion::default(), MemoryRegion::new_from_slice(foo.as_bytes(), va_foo, 0, false), MemoryRegion::new_from_slice(bar.as_bytes(), va_bar, 0, false)], &config).unwrap();
+/// let memory_mapping = MemoryMapping::new::<UserError>(vec![MemoryRegion::default(), MemoryRegion::new_readonly(foo.as_bytes(), va_foo), MemoryRegion::new_readonly(bar.as_bytes(), va_bar)], &config).unwrap();
 /// BpfStrCmp::call(&mut BpfStrCmp {}, va_foo, va_bar, 0, 0, 0, &memory_mapping, &mut result);
 /// assert!(result.unwrap() != 0);
 /// ```
 pub struct BpfStrCmp {}
-impl BpfStrCmp {
-    /// new
-    pub fn init<C, E>(_unused: C) -> Box<dyn SyscallObject<UserError>> {
-        Box::new(Self {})
-    }
-}
 impl SyscallObject<UserError> for BpfStrCmp {
     fn call(
         &mut self,
@@ -328,64 +259,8 @@ impl SyscallObject<UserError> for BpfStrCmp {
 
 // Some additional syscalls
 
-/// Returns a random u64 value comprised between `min` and `max` values (inclusive). Arguments 3 to
-/// 5 are unused.
-///
-/// Relies on `rand()` function from libc, so `libc::srand()` should be called once before this
-/// syscall is used.
-///
-/// # Examples
-///
-/// ```
-/// extern crate libc;
-/// extern crate solana_rbpf;
-///
-/// use solana_rbpf::syscalls::{BpfRand, Result};
-/// use solana_rbpf::memory_region::{MemoryRegion, MemoryMapping};
-/// use solana_rbpf::vm::{Config, SyscallObject};
-/// use solana_rbpf::user_error::UserError;
-//////
-/// let mut result: Result = Ok(0);
-/// let config = Config::default();
-/// let memory_mapping = MemoryMapping::new::<UserError>(vec![], &config).unwrap();
-/// BpfRand::call(&mut BpfRand {}, 3, 6, 0, 0, 0, &memory_mapping, &mut result);
-/// let n = result.unwrap();
-/// assert!(3 <= n && n <= 6);
-/// ```
-pub struct BpfRand {}
-impl BpfRand {
-    /// new
-    pub fn init<C, E>(_unused: C) -> Box<dyn SyscallObject<UserError>> {
-        Box::new(Self {})
-    }
-}
-impl SyscallObject<UserError> for BpfRand {
-    fn call(
-        &mut self,
-        min: u64,
-        max: u64,
-        _arg3: u64,
-        _arg4: u64,
-        _arg5: u64,
-        _memory_mapping: &MemoryMapping,
-        result: &mut Result,
-    ) {
-        let mut n = unsafe { (libc::rand() as u64).wrapping_shl(32) + libc::rand() as u64 };
-        if min < max {
-            n = n % (max + 1 - min) + min;
-        };
-        *result = Result::Ok(n);
-    }
-}
-
 /// Prints a NULL-terminated UTF-8 string.
 pub struct BpfSyscallString {}
-impl BpfSyscallString {
-    /// new
-    pub fn init<C, E>(_unused: C) -> Box<dyn SyscallObject<UserError>> {
-        Box::new(Self {})
-    }
-}
 impl SyscallObject<UserError> for BpfSyscallString {
     fn call(
         &mut self,
@@ -398,7 +273,7 @@ impl SyscallObject<UserError> for BpfSyscallString {
         result: &mut Result,
     ) {
         let host_addr = question_mark!(memory_mapping.map(AccessType::Load, vm_addr, len), result);
-        let c_buf: *const c_char = host_addr as *const c_char;
+        let c_buf: *const i8 = host_addr as *const i8;
         unsafe {
             for i in 0..len {
                 let c = std::ptr::read(c_buf.offset(i as isize));
@@ -416,12 +291,6 @@ impl SyscallObject<UserError> for BpfSyscallString {
 
 /// Prints the five arguments formated as u64 in decimal.
 pub struct BpfSyscallU64 {}
-impl BpfSyscallU64 {
-    /// new
-    pub fn init<C, E>(_unused: C) -> Box<dyn SyscallObject<UserError>> {
-        Box::new(Self {})
-    }
-}
 impl SyscallObject<UserError> for BpfSyscallU64 {
     fn call(
         &mut self,
@@ -444,13 +313,7 @@ impl SyscallObject<UserError> for BpfSyscallU64 {
 /// Example of a syscall with internal state.
 pub struct SyscallWithContext {
     /// Mutable state
-    pub context: BpfSyscallContext,
-}
-impl SyscallWithContext {
-    /// new
-    pub fn init<C, E>(context: BpfSyscallContext) -> Box<dyn SyscallObject<UserError>> {
-        Box::new(Self { context })
-    }
+    pub context: u64,
 }
 impl SyscallObject<UserError> for SyscallWithContext {
     fn call(
