@@ -140,12 +140,23 @@ fn check_jmp_offset(prog: &[u8], insn_ptr: usize) -> Result<(), VerifierError> {
     Ok(())
 }
 
-fn check_registers(insn: &ebpf::Insn, store: bool, insn_ptr: usize) -> Result<(), VerifierError> {
+fn check_registers(
+    insn: &ebpf::Insn,
+    store: bool,
+    insn_ptr: usize,
+    enable_stack_ptr: bool,
+) -> Result<(), VerifierError> {
     if insn.src > 10 {
         return Err(VerifierError::InvalidSourceRegister(adj_insn_ptr(insn_ptr)));
     }
+
     match (insn.dst, store) {
         (0..=9, _) | (10, true) => Ok(()),
+        (11, _)
+            if enable_stack_ptr && (insn.opc == ebpf::SUB64_IMM || insn.opc == ebpf::ADD64_IMM) =>
+        {
+            Ok(())
+        }
         (10, false) => Err(VerifierError::CannotWriteR10(adj_insn_ptr(insn_ptr))),
         (_, _) => Err(VerifierError::InvalidDestinationRegister(adj_insn_ptr(
             insn_ptr,
@@ -167,8 +178,12 @@ fn check_imm_shift(insn: &ebpf::Insn, insn_ptr: usize, imm_bits: u64) -> Result<
 }
 
 /// Check that the imm is a valid register number
-fn check_imm_register(insn: &ebpf::Insn, insn_ptr: usize) -> Result<(), VerifierError> {
-    if insn.imm < 0 || insn.imm > 10 {
+fn check_imm_register(
+    insn: &ebpf::Insn,
+    insn_ptr: usize,
+    config: &Config,
+) -> Result<(), VerifierError> {
+    if insn.imm < 0 || insn.imm > 10 || (insn.imm == 10 && config.reject_callx_r10) {
         return Err(VerifierError::InvalidRegister(adj_insn_ptr(insn_ptr)));
     }
     Ok(())
@@ -238,6 +253,8 @@ pub fn check(prog: &[u8], config: &Config) -> Result<(), VerifierError> {
             ebpf::MUL32_REG  => {},
             ebpf::DIV32_IMM  => { check_imm_nonzero(&insn, insn_ptr)?; },
             ebpf::DIV32_REG  => {},
+            ebpf::SDIV32_IMM if config.enable_sdiv => { check_imm_nonzero(&insn, insn_ptr)?; },
+            ebpf::SDIV32_REG if config.enable_sdiv => {},
             ebpf::OR32_IMM   => {},
             ebpf::OR32_REG   => {},
             ebpf::AND32_IMM  => {},
@@ -267,6 +284,8 @@ pub fn check(prog: &[u8], config: &Config) -> Result<(), VerifierError> {
             ebpf::MUL64_REG  => {},
             ebpf::DIV64_IMM  => { check_imm_nonzero(&insn, insn_ptr)?; },
             ebpf::DIV64_REG  => {},
+            ebpf::SDIV64_IMM if config.enable_sdiv => { check_imm_nonzero(&insn, insn_ptr)?; },
+            ebpf::SDIV64_REG if config.enable_sdiv => {},
             ebpf::OR64_IMM   => {},
             ebpf::OR64_REG   => {},
             ebpf::AND64_IMM  => {},
@@ -310,7 +329,7 @@ pub fn check(prog: &[u8], config: &Config) -> Result<(), VerifierError> {
             ebpf::JSLE_IMM   => { check_jmp_offset(prog, insn_ptr)?; },
             ebpf::JSLE_REG   => { check_jmp_offset(prog, insn_ptr)?; },
             ebpf::CALL_IMM   => {},
-            ebpf::CALL_REG   => { check_imm_register(&insn, insn_ptr)?; },
+            ebpf::CALL_REG   => { check_imm_register(&insn, insn_ptr, config)?; },
             ebpf::EXIT       => {},
 
             _                => {
@@ -318,7 +337,7 @@ pub fn check(prog: &[u8], config: &Config) -> Result<(), VerifierError> {
             }
         }
 
-        check_registers(&insn, store, insn_ptr)?;
+        check_registers(&insn, store, insn_ptr, config.dynamic_stack_frames)?;
 
         insn_ptr += 1;
     }
